\name{dccfit-methods}
\docType{methods}
\alias{dccfit}
\alias{dccfit,ANY-method}
\alias{dccfit,DCCspec-method}
\title{function: DCC-GARCH Fit}
\description{
Method for creating a DCC-GARCH fit object.
}
\usage{
dccfit(spec, data, out.sample = 0, solver = "solnp", solver.control = list(), 
fit.control = list(eval.se = TRUE, stationarity = TRUE, scale = TRUE),
parallel = FALSE, parallel.control = list(pkg = c("multicore", "snowfall"), cores = 2), 
fit = NULL, VAR.fit = NULL, ...)
}
\arguments{
  	\item{spec}{ A \code{\linkS4class{DCCspec}} object created by calling \sQuote{dccspec}.}
  	\item{data}{ A multivariate data object as \sQuote{matrix} or \sQuote{data.frame}.}
  	\item{out.sample}{ A positive integer indicating the number of periods before the last
	to keep for out of sample forecasting.}
  	\item{solver}{ Either \dQuote{nlminb} or \dQuote{solnp} .}
 	\item{solver.control}{Control arguments list passed to optimizer.}
	\item{fit.control}{Control arguments passed to the fitting routine. The \sQuote{eval.se} option
	determines whether standard errors are calculated (see details below). The \sQuote{stationarity} 
	option is for the univariate stage GARCH fitting routine, whilst for the second stage DCC this is 
	be design imposed. The \sQuote{scale} option is also for the first stage univariate GARCH fitting routine.}
	\item{parallel}{ Whether to make use of parallel processing on multicore systems.}
    \item{parallel.control}{ The parallel control options including the type of package for 
  	performing the parallel calculations (\sQuote{multicore} for non-windows O/S and 
  	\sQuote{snowfall} for all O/S), and the number of cores to make use of.}
	\item{fit}{ (optional) A previously estimated univariate \code{\linkS4class{uGARCHmultifit}} object
	(see details).}
	\item{VAR.fit}{ (optional) A previously estimated VAR list returned from calling the 
	\code{\link{varxfilter}} function.}
	\item{...}{ . }
}
\value{
A \code{\linkS4class{DCCfit}} object containing details of the DCC-GARCH fit.
}
\author{Alexios Ghalanos}
\details{
The 2-step DCC estimation fits a GARCH-Normal model to the univariate data and then proceeds to
estimate the second step based on the chosen multivariate distribution. Because of this 2-step approach,
standard errors are expensive to calculate and therefore the use of parallel functionality, built into
both the fitting and standard error calculation routines is key. The switch to turn off the calculation
of standard errors through the \sQuote{fit.control} option could be quite useful in rolling estimation
such as in the \code{\link{dccroll}} routine.\cr
The optional \sQuote{fit} argument allows to pass your own \code{\linkS4class{uGARCHmultifit}} object
instead of having the routine estimate it. This is very useful in cases of multiple use of the same
fit and problems in convergence which might require a more hands on approach to the univariate fitting
stage. However, it is up to the user to ensure consistency between the \sQuote{fit} and supplied
\sQuote{spec}. Also note that the \sQuote{dccfit} method will assign to the global environment the
\code{\linkS4class{uGARCHmultifit}} once that is estimated in order to allow the routine to be 
restarted should something go wrong (it should show up as \sQuote{.fitlist}).
}
\note{
There is no check on the VAR.fit list passed to the method so particular care should be exercised so that the same data used 
in the fitting routine is also used in the VAR filter routine. The ability to pass this list of the pre-calculated VAR model is 
particularly useful when comparing different models (such as copula GARCH, GOGARCH etc) using the same dataset and VAR method. 
Though the classical VAR estimation is very fast and may not require this extra step, the robust method is slow and therefore 
benefits from calculating this only once.
}
\examples{
\dontrun{
data(dji30ret)
Dat = dji30ret[, 1:4, drop = FALSE]
uspec1 = ugarchspec(mean.model = list(armaOrder = c(1,0)), variance.model = list(model = "apARCH"), 
			distribution.model = "norm")
uspec2 = ugarchspec(mean.model = list(armaOrder = c(2,0)), variance.model = list(model = "gjrGARCH"), 
			distribution.model = "norm")
uspec3 = ugarchspec(mean.model = list(armaOrder = c(0,0)), variance.model = list(model = "sGARCH"), 
			distribution.model = "norm")
uspec4 = ugarchspec(mean.model = list(armaOrder = c(1,0)), variance.model = list(model = "sGARCH",
					garchOrder = c(2, 1)), 
			distribution.model = "norm")
uspec = c( uspec1, uspec2, uspec3, uspec4 )
spec = dccspec(uspec = multispec( uspec ), dccOrder = c(1,1), distribution = "mvlaplace")
# fit the model
fit = dccfit(data = Dat, spec = spec, out.sample = 100, solver = "solnp", 
fit.control = list(scale = TRUE, eval.se = TRUE))
# show the fit
fit
# plot(fit)
}}
\keyword{methods}