\name{dccroll-methods}
\docType{methods}
\alias{dccroll}
\alias{dccroll,ANY-method}
\alias{dccroll,DCCspec-method}
\title{function: DCC-GARCH Rolling Forecast}
\description{
Method for creating a DCC-GARCH rolling forecast object.
}
\usage{
dccroll(spec, data, n.ahead = 1, forecast.length = 50, refit.every = 25, 
refit.window = c("recursive", "moving"), solver = "solnp", fit.control = list(eval.se = TRUE),  
parallel = FALSE, parallel.control = list(pkg = c("multicore", "snowfall"), cores = 2), 
solver.control = list(), save.fit = FALSE, save.wdir = NULL, trace = FALSE, ...)
}
\arguments{
	\item{spec}{ A \code{\linkS4class{DCCspec}} object with fixed parameters.}
	\item{data}{ A multivariate dataset as \sQuote{matrix} or \sQuote{data.frame}.}
	\item{n.ahead}{ The number of periods to forecast. }
  	\item{forecast.length}{ The length of the total forecast for which out of sample data from 
  	the dataset will be excluded for testing.}
  	\item{refit.every}{ Determines every how many periods the model is re-estimated.}
  	\item{refit.window}{ Whether the refit is done on an expanding window including all the 
  	previous data or a moving window, the length of the window determined by the argument above 
  	(\dQuote{refit.every}).}
	\item{parallel}{ Whether to make use of parallel processing on multicore systems.}
    \item{parallel.control}{ The parallel control options including the type of package for 
  	performing the parallel calculations (\sQuote{multicore} for non-windows O/S and 
  	\sQuote{snowfall} for all O/S), and the number of cores to make use of.}
  	\item{solver}{ The solver to use.}
  	\item{fit.control}{ Control parameters parameters passed to the fitting function.}
  	\item{solver.control}{ Control parameters passed to the solver.}
    \item{save.fit}{ Whether to save the fitted objects of class \code{\linkS4class{DCCfit}} 
    during the estimation of each (\dQuote{refit.every}). If true, the directory to save must 
    be provided (see below). The function will not save this by default for reasons of 
    memory management, but can save it as an \dQuote{.rda} file in the user's chosen 
    directory for further analysis.}
  	\item{save.wdir}{ If \dQuote{save.fit} is true, the directory in which to save the 
  	\code{\linkS4class{DCCfit}} objects (1 for each \dQuote{refit.every}).}
  	\item{trace}{ Traces out each roll iteration. }
  	\item{\dots}{.}
}
\value{
A \code{\linkS4class{DCCroll}} object containing details of the DCC-GARCH forecast.
}
\examples{
\dontrun{
data(dji30ret)
Dat = dji30ret[, 1:5, drop = FALSE]
uspec1 = ugarchspec(mean.model = list(armaOrder = c(1,0)), variance.model = list(model = "apARCH"), 
		distribution.model = "norm")
uspec2 = ugarchspec(mean.model = list(armaOrder = c(2,0)), variance.model = list(model = "gjrGARCH"), 
		distribution.model = "norm")
uspec3 = ugarchspec(mean.model = list(armaOrder = c(0,0)), variance.model = list(model = "sGARCH"), 
		distribution.model = "norm")
uspec4 = ugarchspec(mean.model = list(armaOrder = c(1,0)), variance.model = list(model = "sGARCH",
		garchOrder = c(2, 1)), distribution.model = "norm")
uspec5 = ugarchspec(mean.model = list(armaOrder = c(1,1)), variance.model = list(model = "eGARCH",
		garchOrder = c(1, 1)), distribution.model = "norm")

uspec = c( uspec1, uspec2, uspec3, uspec4, uspec5 )
spec = dccspec(uspec = multispec( uspec ), dccOrder = c(1,1), distribution = "mvt")

roll = dccroll(spec, data = Dat, n.ahead = 1, forecast.length = 100, refit.every = 25, 
		refit.window = "moving", solver = "solnp", 
		fit.control = list(eval.se = FALSE), solver.control = list(), 
		save.fit = FALSE, save.wdir = NULL, trace = TRUE)

roll
cat("\nAverage Log-Likelihood Across Rolls:\n")
print( round( likelihood(object)/object@spec$n.data, 2) )

cat("\nCorrelation from roll object\n")
cat("\nis(rcor(roll, roll = 1))[1]\n")
is(rcor(roll))[1]
cat("\nThere are 4 rolls (forecast.length/refit.every).")
cat("\nEach rolling window correlation is a list of arrays of Length = refit.every,")
cat("\nwith each array equal to m x m (no. assets) by n.ahead.\n")
cat("\nSince n.ahead = 1, reshape list to array:.")
cat("\narray(unlist(rcor(roll, roll = 1)) , dim = c(5,5,25))\n")
array(unlist(rcor(roll)) , dim = c(5,5,25))
}}
\author{Alexios Ghalanos}
\keyword{methods}