\name{dccsim-methods}
\docType{methods}
\alias{dccsim}
\alias{dccsim,ANY-method}
\alias{dccsim,DCCspec-method}
\alias{dccsim,DCCfit-method}
\title{function: DCC-GARCH Simulation}
\description{
Method for creating a DCC-GARCH simulation object.
}
\usage{
dccsim(fitORspec, n.sim = 1000, n.start = 0, m.sim = 1, startMethod = c("unconditional", "sample"), 
		presigma = NULL, preresiduals = NULL, prereturns = NULL, preR = NULL, preH = NULL, rseed = NULL, 
		mexsimdata = NULL, vexsimdata = NULL, parallel = FALSE, 
		parallel.control = list(pkg = c("multicore", "snowfall"), cores = 2), VAR.fit = NULL, ...)
}
\arguments{
  	\item{fitORspec}{ A \code{\linkS4class{DCCspec}} or \code{\linkS4class{DCCfit}} object created 
  	by calling either \code{\link{dccspec}} with fixed parameters or \code{\link{dccfit}}.}
	\item{n.sim}{ The simulation horizon.}
	\item{n.start}{ The burn-in sample.}
	\item{m.sim}{ The number of simulations.}
	\item{startMethod}{ Starting values for the simulation. Valid methods are 
	\sQuote{unconditional} for the expected values given the density, and \sQuote{sample} 
	for the ending values of the actual data from the fit object.}
	\item{presigma}{ Allows the starting sigma values to be provided by the user.}
	\item{prereturns}{ Allows the starting return data to be provided by the user.}
	\item{preresiduals}{ Allows the starting residuals to be provided by the user.}
	\item{preR}{ Allows the starting correlation to be provided by the user.}
	\item{preH}{ Allows the starting covariance to be provided by the user.}
	\item{rseed}{ Optional seeding value(s) for the random number generator. For m.sim>1, it is possible
	to provide either a single seed to initialize all values, or one seed per separate simulation 
	(i.e. m.sim seeds). However, in the latter case this may result in some slight overhead depending on
	how large m.sim is.}
	\item{mexsimdata}{ A list (equal to the number of asset) of matrices of simulated external 
	regressor-in-mean data with row length equal to n.sim + n.start. If the fit object 
	contains external regressors in the mean equation, this must be provided else will be assumed to be zero.}
	\item{vexsimdata}{ A list (equal to the number of asset) of matrices of simulated external 
	regressor-in-variance data with row length equal to n.sim + n.start. If the fit object contains 
	external regressors in the variance equation, this must be provided else will be assumed to be zero.}
	\item{parallel}{ Whether to make use of parallel processing on multicore systems.}
    \item{parallel.control}{ The parallel control options including the type of package for 
  	performing the parallel calculations (\sQuote{multicore} for non-windows O/S and 
  	\sQuote{snowfall} for all O/S), and the number of cores to make use of.}
	\item{VAR.fit}{ An optional VAR.fit list returned from calling the \code{\link{varxfilter}} function.
	This is for the dispatch method with \code{\linkS4class{DCCspec}} object since the \code{\linkS4class{DCCfit}} 
	object will already contain all necessary information for the VAR model.}
  	\item{...}{.}
}
\value{
A \code{\linkS4class{DCCsim}} object containing details of the DCC-GARCH simulation.
}
\details{
In order to pass a correct spec to the filter routine, you must ensure that it contains the
appropriate \sQuote{fixed.pars} in both the multivariate DCC part of the spec as well
as the multiple univariate spec part. An illustrative example is contained below.
}
\author{Alexios Ghalanos}
\examples{
\dontrun{
data(dji30ret)
Dat = dji30ret[, 1:4, drop = FALSE]
uspec1 = ugarchspec(mean.model = list(armaOrder = c(1,0)), variance.model = list(model = "apARCH"), 
		distribution.model = "norm")
uspec2 = ugarchspec(mean.model = list(armaOrder = c(2,0)), variance.model = list(model = "gjrGARCH"), 
		distribution.model = "norm")
uspec3 = ugarchspec(mean.model = list(armaOrder = c(0,0)), variance.model = list(model = "sGARCH"), 
		distribution.model = "norm")
uspec4 = ugarchspec(mean.model = list(armaOrder = c(1,0)), variance.model = list(model = "sGARCH",
		garchOrder = c(2, 1)), distribution.model = "norm")

uspec = c( uspec1, uspec2, uspec3, uspec4 )
spec = dccspec(uspec = multispec( uspec ), dccOrder = c(1,1), distribution = "mvt")

# fit the model
fit = dccfit(data = Dat, spec = spec, out.sample = 100, solver = "solnp", 
	fit.control = list(scale = TRUE, eval.se = FALSE))

# simulation takes either fit or spec (check both)
spec2 = spec
# fix the dcc parameters
spec2@mspec$optimization.model$fixed.pars = coef(fit, type = "dcc")
# fix the garchpars
for(i in 1:4) spec2@uspec@spec[[i]]@optimization.model$fixed.pars = coef(fit@ufit@fit[[i]])

preR = cor(as.matrix(Dat))
presigma = tail( sigma(fit@ufit), 2 )
preresiduals = tail( residuals(fit@ufit), 2 )
prereturns = tail( as.matrix(Dat), 2 )

# also possible to obtain uncvariance by supplying parameters
# showMethods("uncvariance")
uncv = sapply(fit@ufit@fit, FUN = function(x) uncvariance(x))
preH = diag(sqrt(uncv)) %*% preR %*% diag(sqrt(uncv))
		
sim1 = dccsim(fitORspec = fit, n.sim = 1000, n.start = 100, m.sim = 2, startMethod = "sample", 
	presigma = presigma, preresiduals = preresiduals, prereturns = prereturns, preR = preR, preH = NULL, 
	rseed = c(100, 200), mexsimdata = NULL, vexsimdata = NULL)

sim2 = dccsim(fitORspec = spec2, n.sim = 1000, n.start = 100, m.sim = 2, 
	presigma = presigma, preresiduals = preresiduals, prereturns = prereturns, preR = preR, preH = preH, 
	rseed = c(100, 200), mexsimdata = NULL, vexsimdata = NULL)

R1 = rcor(sim1, sim = 1)
R2 = rcor(sim2, sim = 1)
H1 = rcov(sim1, sim = 2)
H2 = rcov(sim2, sim = 2)

# now replicate the conditions for the sample start
sim3 = dccsim(fitORspec = fit, n.sim = 1000, n.start = 100, m.sim = 2, startMethod = "sample", 
		presigma = NULL, preresiduals = NULL, prereturns = NULL, preR = preR, preH = NULL, 
		rseed = c(100, 200), mexsimdata = NULL, vexsimdata = NULL)

# prereturns should be NULL
sim4 = dccsim(fitORspec = spec2, n.sim = 1000, n.start = 100, m.sim = 2, 
		presigma = presigma, preresiduals = preresiduals, prereturns = NULL, preR = preR, preH = preH, 
		rseed = c(100, 200), mexsimdata = NULL, vexsimdata = NULL)

R3 = rcor(sim3, sim = 2)
R4 = rcor(sim4, sim = 2)
H3 = rcor(sim3, sim = 1)
H4 = rcor(sim4, sim = 1)

cat("\nDCC Simulation Tests:\n")
cat("\nFit and Spec based Simulation DCC model Conditional Correlation check1:\n")
cat("\nn = 1\n")
print(R1[, , 1] == R2[, , 1])
cat("\nn = 100\n")
print(R1[, , 100] == R2[, , 100])
cat("\nFit and Spec based Simulation DCC model Conditional Covariance check1:\n")
cat("\nn = 1\n")
print(H1[, , 1] == H2[, , 1])
cat("\nn = 100\n")
print(H1[, , 100] == H2[, , 100])
cat("\nFit and Spec based Simulation DCC model Conditional Correlation check2:\n")
cat("\nn = 1\n")
print(R3[, , 1] == R4[, , 1])
cat("\nn = 100\n")
print(R3[, , 100] == R4[, , 100])
cat("\nFit and Spec based Simulation DCC model Conditional Covariance check2:\n")
cat("\nn = 1\n")
print(H3[, , 1] == H4[, , 1])
cat("\nn = 100\n")
print(H3[, , 100] == H4[, , 100])
}}
\keyword{methods}